/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */

var game;
var cursors;
var bEndGame = false;// fin du jeu ?

var FONT = 32;// taille de la police de caractères

// La taille de la carte du jeu
var ROWS = 15;
var COLS = 33;

var oBloc = '|';// obstacle
var oEnnemis = 'O';// les ennemis


var ACTORS = 29;// nombre de joueur sur la carte, héros inclus
var aMap;// le tableau contenant les données du jeu

var aScreen;// l'écran de jeu en 2D avec des caractères

// liste des joueurs, 0 est le héros
var player;
var nXplayer = 0;
var nYplayer = 0;


var actorList;
var livingEnemies;// nombre d'ennemis en vie

var actorMap;// objet de chaque joueur, pour les repérer facilement sur le jeu
var isMovingPlayer = false;// est-ce que le joueur est entrain de se déplacer


document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API Canvas
    game = new Phaser.Game(COLS * FONT * 0.6, ROWS * FONT, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});
}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    game.stage.backgroundColor = '#cccccc';// la couleur de fond du jeu
}



/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{
    // écoute les touches directionnelles pressées par le joueur
    cursors = game.input.keyboard.createCursorKeys();// controle du héros avec les touches directionnelles

    // capture les actions sur les touches ci-dessous
    game.input.keyboard.addKeyCapture([
        Phaser.Keyboard.LEFT,
        Phaser.Keyboard.RIGHT,
        Phaser.Keyboard.UP,
        Phaser.Keyboard.DOWN
    ]);

    initMap();// initialise la carte du jeu

    // initialise l'écran de jeu
    aScreen = [];
    for (var y = 0; y < ROWS; y++) 
    {
        var newRow = [];
        aScreen.push(newRow);// ajoute les colonnes

        for (var x = 0; x < COLS; x++)
            newRow.push(initCell('', x, y));
    }
    
    initActors();// initialise les joueurs

    // dessine le niveau
    drawMap();
    drawActors();
}


/**
 * Initialise les cases du jeu avec le contenu
 * 
 * @param  {[type]} sString [description]
 * @param  {[type]} x   [description]
 * @param  {[type]} y   [description]
 * @return {[type]}     [description]
 */
function initCell(sString, x, y) 
{
    // ajoute une case à l'écran à la position donnée
    var style = {
        font: FONT + "px monospace",
        fill: "#6666DD"
    };
    return game.add.text(FONT * 0.6 * x, FONT * y, sString, style);
}


/**
 * Préparation de la carte du jeu
 * 
 * @return {[type]} [description]
 */
function initMap() 
{
    // Création d'une nouvelle carte aléatoire
    aMap = [];

    for (var y = 0; y < ROWS; y++) 
    {
        var newRow = [];
        for (var x = 0; x < COLS; x++) 
        {
            if (Math.random() > 0.8) 
                newRow.push(oBloc);
            else 
                newRow.push('.');
        }

        aMap.push(newRow);
    }
}


/**
 * Dessine la carte du jeu
 * (ajoute tous les joueurs)
 * 
 * @return {[type]} [description]
 */
function drawMap()
{
    for (var y = 0; y < ROWS; y++)
        for (var x = 0; x < COLS; x++)
            aScreen[y][x].content = aMap[y][x];
}


/**
 * Renvoit un nombre aléatoire
 * 
 * @param  {[type]} max [description]
 * @return {[type]}     [description]
 */
function randomInt(nMaxValue)
{
    return Math.floor(Math.random() * nMaxValue);
}


/**
 * Iniatilisation des joueurs
 * 
 * @return {[type]} [description]
 */
function initActors()
{
    // ajoute les joueurs à des positions aléatoires
    actorList = [];
    actorMap = {};

    for (var e = 0; e < ACTORS; e++)
    {
        // création d'un nouveau joueur
        var actor =
        {
            x: 0,
            y: 0,
            hp: e == 0 ? 3 : 1// le nombre de vies (héros -> 3, ennemis -> 1)
        };

        do
        {
            // détermine une position aléatoire et vide
            actor.y = randomInt(ROWS);
            actor.x = randomInt(COLS);
        } while (aMap[actor.y][actor.x] == oBloc || actorMap[actor.y + "_" + actor.x] != null);


        // add references to the actor to the actors list & map
        actorMap[actor.y + "_" + actor.x] = actor;
        actorList.push(actor);
    }

    // le héros est le 1er joueur de la liste
    player = actorList[0];
    livingEnemies = ACTORS - 1;
}


/**
 * Mets en place les ennemis
 * 
 * @return {[type]} [description]
 */
function drawActors()
{
    for (var a in actorList)
    {
        if (actorList[a] != null && actorList[a].hp > 0) 
            aScreen[actorList[a].y][actorList[a].x].content = a == 0 ? '' + player.hp : oEnnemis;
    }
}


/**
 * Est ce que le joueur peut se déplacer ici ?
 * 
 * @param  {[type]} actor [description]
 * @param  {[type]} dir   [description]
 * @return {[type]}       [description]
 */
function canGo(actor,dir)
{
    return  actor.x+dir.x >= 0 &&
            actor.x+dir.x <= COLS - 1 &&
            actor.y+dir.y >= 0 &&
            actor.y+dir.y <= ROWS - 1 &&
            aMap[actor.y+dir.y][actor.x +dir.x] == '.';
}


/**
 * Déplacement d'un joueur
 * 
 * @param  {[type]} actor [description]
 * @param  {[type]} dir   [description]
 * @return {[type]}       [description]
 */
function moveTo(actor, dir)
{
    // vérifie si le joueur peut se déplacer dans cette direction
    if (!canGo(actor,dir)) 
        return false;
    
    
    var newKey = (actor.y + dir.y) +'_' + (actor.x + dir.x);// déplace le joueur dans sa nouvelle position
    
    // si la destination a déjà un joueur et qu'il y a une collision avec le héros
    if (actorMap[newKey] != null)
    {
        // décrémente le nombre de point du joueur
        var oVictim = actorMap[newKey];
        oVictim.hp--;
        
        // si le joueur est mort --> suppression
        if (oVictim.hp == 0)
        {
            actorMap[newKey]= null;
            actorList[actorList.indexOf(oVictim)]=null;

            if(oVictim!=player)
            {
                livingEnemies--;

                // si tous les ennemis sont morts
                if (livingEnemies == 0)
                {
                    // c'est gagné
                    var victory = game.add.text(game.world.centerX, game.world.centerY, 'Victoire !!!', { fill : '#2e2', align: "center" } );
                    victory.anchor.setTo(0.5,0.5);

                    bEndGame = true;
                }
            }
        }
    }
    else 
    {
        // supprime l'ancienne position du joueur
        actorMap[actor.y + '_' + actor.x]= null;
        
        // mets à jour sa position
        actor.y+=dir.y;
        actor.x+=dir.x;

        // ajoute une référence du joueur sur sa nouvelle position
        actorMap[actor.y + '_' + actor.x]=actor;
    }
    return true;
}



/**
 * Le joueur peut se déplacer
 * 
 * @param  {[type]} player [description]
 * @param  {[type]} star   [description]
 * @return {[type]}        [description]
 */
function movePlayerGo(nX, nY)
{
    // si le joueur ne déplace pas déjà --> active le déplacement
    if (! isMovingPlayer)
    {
        isMovingPlayer = true;
        nXplayer = nX;
        nYplayer = nY;
    }    
}



/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
    // si le jeu est terminé
    if(bEndGame == true)
        return;

    drawMap();// dessine les joueurs dans leurs nouvelles positions

    if( cursors.left.isDown)
        movePlayerGo(-1, 0);
    else if( cursors.right.isDown)
        movePlayerGo(1, 0);
    else if( cursors.up.isDown)
        movePlayerGo(0, -1);
    else if( cursors.down.isDown)
        movePlayerGo(0, 1);
    
    /*console.log("movePlayers, X : "+nXplayer);
    console.log("Y : "+nYplayer);*/

    // déplace le joueur
    if( isMovingPlayer == true )
    {
        var acted = false;
        acted = moveTo(player, {x:nXplayer, y:nYplayer});

        // enemies act every time the player does
        if (acted)
        {
            for (var enemy in actorList)
            {
                // saute le héros
                if(enemy==0)
                    continue;
                
                var e = actorList[enemy];
                
                if (e != null)
                    moveEnnemis(e);
            }

        }

        isMovingPlayer = false;
    }
   
    drawActors();// dessine les joueurs dans leurs nouvelles positions

    return;
}


/**
 * Déplace les ennemis
 * 
 * @param  {[type]} actor [description]
 * @return {[type]}       [description]
 */
function moveEnnemis(actor)
{
    var directions = [ { x: -1, y:0 }, { x:1, y:0 }, { x:0, y: -1 }, { x:0, y:1 } ];    
    var dx = player.x - actor.x;
    var dy = player.y - actor.y;
    

    if (Math.abs(dx) + Math.abs(dy) > 6)
    {
        var iCpt = 0;
        while ( !moveTo(actor, directions[randomInt(directions.length)]) && iCpt < 3000)
        {
            iCpt++;
        };
    }   
    

    if (Math.abs(dx) > Math.abs(dy))
    {
        if (dx < 0)  
            moveTo(actor, directions[0]);// vers la gauche
        else 
            moveTo(actor, directions[1]);// vers la droite
    }
    else
    {
        if (dy < 0)
            moveTo(actor, directions[2]);// vers le haut
        else
            moveTo(actor, directions[3]);// vers le bas
    }

    // si le héros n'a plus de vie...
    if (player.hp < 1)
    {
        // fin de la partie
        var gameOver = game.add.text(game.world.centerX, game.world.centerY, 'Game Over !!!', { fill : '#e22', align: "center" } );
        gameOver.anchor.setTo(0.5,0.5);

        bEndGame = true;
    }
}
